@@ -14,7 +14,6 @@ from typing import TYPE_CHECKING, Any, cast
 
 import pathspec
 import wcmatch.pathlib
-import wcmatch.wcmatch
 from yaml.error import YAMLError
 
 from ansiblelint.app import get_app
@@ -169,7 +168,7 @@ def kind_from_path(path: Path, *, base: bool = False) -> FileType:
     if str(path) == "/dev/stdin":
         return "playbook"
 
-    if "pattern.json" in str(path):
+    if path.name == "pattern.json":
         # This is a pattern file
         return "pattern"
 
